package builder

import (
	"context"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/endpoints"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/model"
	"strconv"
)

// TagsBuilder represents a builder for constructing queries to retrieve tags.
type TagsBuilder interface {
	// SearchName sets the query to search for tags with specific names.
	SearchName(name string) TagsBuilder
	// SearchCategory sets the query to search for tags in a specific category.
	SearchCategory(category model.TagCategory) TagsBuilder
	// Order sets the query to order tags by a specific criterion.
	Order(order string) TagsBuilder
	// HideEmpty sets the query to filter out tags that are empty.
	HideEmpty(hideEmpty bool) TagsBuilder
	// Wiki sets the query to filter tags that have a wiki.
	Wiki(hasWiki bool) TagsBuilder
	// Artist sets the query to filter tags that have an artist page.
	Artist(hasArtistPage bool) TagsBuilder
	// SetPage sets the query to retrieve tags from a specific page number.
	SetPage(pageNumber int) TagsBuilder
	// SetLimit sets the maximum number of tags to retrieve.
	SetLimit(limitUser int) TagsBuilder
	// Execute sends the constructed query and returns a slice of tags and an error, if any.
	Execute() ([]model.Tag, error)
}

// NewGetTagsBuilder creates a new TagsBuilder with the provided RequestContext.
func NewGetTagsBuilder(requestContext model.RequestContext) TagsBuilder {
	return &getTags{requestContext: requestContext, query: make(map[string]string)}
}

type getTags struct {
	requestContext model.RequestContext
	query          map[string]string
}

// SearchName sets the query to search for tags with specific names.
func (g *getTags) SearchName(name string) TagsBuilder {
	g.query["search[name_matches]"] = name
	return g
}

// SearchCategory sets the query to search for tags in a specific category.
func (g *getTags) SearchCategory(category model.TagCategory) TagsBuilder {
	g.query["search[category]"] = strconv.Itoa(int(category))
	return g
}

// Order sets the query to order tags by a specific criterion.
func (g *getTags) Order(order string) TagsBuilder {
	g.query["search[order]"] = order
	return g
}

// HideEmpty sets the query to filter out tags that are empty.
func (g *getTags) HideEmpty(hideEmpty bool) TagsBuilder {
	g.query["search[hide_empty]"] = strconv.FormatBool(hideEmpty)
	return g
}

// Wiki sets the query to filter tags that have a wiki.
func (g *getTags) Wiki(hasWiki bool) TagsBuilder {
	g.query["search[has_wiki]"] = strconv.FormatBool(hasWiki)
	return g
}

// Artist sets the query to filter tags that have an artist page.
func (g *getTags) Artist(hasArtistPage bool) TagsBuilder {
	g.query["search[has_artist]"] = strconv.FormatBool(hasArtistPage)
	return g
}

// SetPage sets the query to retrieve tags from a specific page number.
func (g *getTags) SetPage(pageNumber int) TagsBuilder {
	g.query["page"] = strconv.Itoa(pageNumber)
	return g
}

// SetLimit sets the maximum number of tags to retrieve.
func (g *getTags) SetLimit(limitUser int) TagsBuilder {
	g.query["limit"] = strconv.Itoa(limitUser)
	return g
}

// Execute sends the constructed query and returns a slice of tags and an error, if any.
func (g *getTags) Execute() ([]model.Tag, error) {
	if g.requestContext.RateLimiter != nil {
		err := g.requestContext.RateLimiter.Wait(context.Background())
		if err != nil {
			return nil, err
		}
	}
	tags, err := endpoints.GetTags(g.requestContext, g.query)
	if err != nil {
		return nil, err
	}
	return tags, err
}
