package endpoints

import (
	"encoding/json"
	"fmt"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/model"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/utils"
	"io"
	"net/http"
	"strings"
)

// GetNote retrieves a single note by its ID from the e621 API.
//
// Parameters:
//   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - ID: The ID of the note to retrieve.
//
// Returns:
//   - model.Note: The retrieved note.
//   - error: An error, if any, encountered during the API request or response handling.
func GetNote(requestContext model.RequestContext, ID string) (model.Note, error) {
	// Create a new HTTP GET request to fetch the note information.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/notes/%s.json", requestContext.Host, ID), nil)
	if err != nil {
		// Log the error and return an empty Note struct and the error.

		return model.Note{}, err
	}

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application/json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		// Log the error and return an empty Note struct and the error.

		return model.Note{}, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return model.Note{}, utils.StatusCodesToError(resp.StatusCode)
	}

	// Initialize a Note struct to store the response data.
	var noteResponse model.Note

	// Decode the JSON response into the Note struct.
	err = json.NewDecoder(resp.Body).Decode(&noteResponse)
	if err != nil {
		// Log the error and return an empty Note struct and the error.

		return model.Note{}, err
	}

	// Return the note information and no error (nil).
	return noteResponse, nil
}

// GetNotes retrieves a list of notes from the e621 API based on query parameters.
//
// Parameters:
//   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - query: A map containing additional query parameters for the API request.
//
// Returns:
//   - []model.Note: A slice of notes.
//   - error: An error, if any, encountered during the API request or response handling.
func GetNotes(requestContext model.RequestContext, query map[string]string) ([]model.Note, error) {
	// Create a new HTTP GET request.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/notes.json", requestContext.Host), nil)
	if err != nil {
		return nil, err
	}

	// Append query parameters to the request URL.
	q := r.URL.Query()
	for k, v := range query {
		q.Add(k, v)
	}
	r.URL.RawQuery = q.Encode()

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application/json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		return nil, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return nil, utils.StatusCodesToError(resp.StatusCode)
	}

	respBodyBytes, err := io.ReadAll(resp.Body)
	if strings.Contains(string(respBodyBytes), "{\"notes\":[]}") {
		return nil, nil
	}

	// Initialize a slice of Note struct to store the response data.
	var notesResponse []model.Note

	// Decode the JSON response into the slice of Note structs.
	err = json.Unmarshal(respBodyBytes, &notesResponse)
	if err != nil {
		// Log the error and return an empty slice and the error.

		return nil, err
	}

	// Return the list of notes and no error (nil).
	return notesResponse, nil
}
