package endpoints

import (
	"encoding/json"
	"fmt"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/model"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/utils"
	"net/http"
)

// GetPost retrieves a single post by its ID from the e621 API.
//
// Parameters: //   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - ID: The ID of the post to retrieve.
//
// Returns:
//   - model.Post: The retrieved post.
//   - error: An error, if any, encountered during the API request or response handling.
func GetPost(requestContext model.RequestContext, ID string) (model.Post, error) {
	// Create a new HTTP GET request to fetch the post information.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/posts/%s.json", requestContext.Host, ID), nil)
	if err != nil {
		// Log the error and return an empty Post struct and the error.

		return model.Post{}, err
	}

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application/json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		// Log the error and return an empty Post struct and the error.

		return model.Post{}, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return model.Post{}, utils.StatusCodesToError(resp.StatusCode)
	}

	// Initialize a Post struct to store the response data.
	var postResponse model.PostResponse

	// Decode the JSON response into the PostResponse struct.
	err = json.NewDecoder(resp.Body).Decode(&postResponse)
	if err != nil {
		// Log the error and return an empty Post struct and the error.

		return model.Post{}, err
	}

	// Return the post information and no error (nil).
	return postResponse.Post, nil
}

// GetPosts retrieves a list of posts from the e621 API based on query parameters.
//
// Parameters:
//   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - query: A map containing additional query parameters for the API request.
//
// Returns:
//   - []model.Post: A slice of posts.
//   - error: An error, if any, encountered during the API request or response handling.
func GetPosts(requestContext model.RequestContext, query map[string]string) ([]model.Post, error) {
	// Create a new HTTP GET request.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/posts.json", requestContext.Host), nil)
	if err != nil {
		return nil, err
	}

	// Append query parameters to the request URL.
	q := r.URL.Query()
	for k, v := range query {
		q.Add(k, v)
	}
	r.URL.RawQuery = q.Encode()

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application/json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		return nil, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return nil, utils.StatusCodesToError(resp.StatusCode)
	}

	// Initialize a slice of Post struct to store the response data.
	var postResponse model.PostResponse

	// Decode the JSON response into the PostResponse struct.
	err = json.NewDecoder(resp.Body).Decode(&postResponse)
	if err != nil {
		// Log the error and return an empty slice and the error.

		return nil, err
	}

	// Return the list of posts and no error (nil).
	return postResponse.Posts, nil
}
