package endpoints

import (
	"encoding/json"
	"fmt"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/model"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/utils"
	"net/http"
)

// GetTag retrieves a tag by its ID from the e621 API.
//
// Parameters:
//   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - ID: The ID of the tag to retrieve.
//
// Returns:
//   - model.Tag: The retrieved tag.
//   - error: An error, if any, encountered during the API request or response handling.
func GetTag(requestContext model.RequestContext, ID string) (model.Tag, error) {
	// Create a new HTTP GET request to fetch tag information.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/tags/%s.json", requestContext.Host, ID), nil)
	if err != nil {
		// Log the error and return an empty Tag struct and the error.

		return model.Tag{}, err
	}

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application.json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		// Log the error and return an empty Tag struct and the error.

		return model.Tag{}, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return model.Tag{}, utils.StatusCodesToError(resp.StatusCode)
	}

	// Initialize a Tag struct to store the response data.
	var tag model.Tag

	// Decode the JSON response into the Tag struct.
	err = json.NewDecoder(resp.Body).Decode(&tag)
	if err != nil {
		// Log the error and return an empty Tag struct and the error.

		return model.Tag{}, err
	}

	// Return the tag information and no error (nil).
	return tag, nil

}

// GetTags retrieves a list of tags from the e621 API based on query parameters.
//
// Parameters:
//   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - query: A map containing additional query parameters for the API request.
//
// Query:
//   - search[name_matches]: A tag name expression to match against, which can include * as a wildcard.
//   - search[category]: Filters results to a particular category. Default value is blank (show all tags). 0 general; 1 artist; 3 copyright; 4 character; 5 species; 6 invalid; 7 meta; 8 lore
//   - search[order]: Changes the sort order. Pass one of date (default), count, or name.
//   - search[hide_empty]: Hide tags with zero visible posts. Pass true (default) or false.
//   - search[has_wiki]: Show only tags with, or without, a wiki page. Pass true, false, or blank (default).
//   - search[has_artist]: Show only tags with, or without an artist page. Pass true, false, or blank (default).
//   - limit: Maximum number of results to return per query. Default is 75. There is a hard upper limit of 320.
//   - page: The page that will be returned. Can also be used with a or b + tag_id to get the tags after or before the specified tag ID. For example a13 gets every tag after tag_id 13 up to the limit. This overrides the specified search ordering, date is always used instead.
//
// Returns:
//   - []model.Tag: A slice of tags.
//   - error: An error, if any, encountered during the API request or response handling.
func GetTags(requestContext model.RequestContext, query map[string]string) ([]model.Tag, error) {
	// Create a new HTTP GET request.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/tags.json", requestContext.Host), nil)
	if err != nil {
		return nil, err
	}

	// Append query parameters to the request URL.
	q := r.URL.Query()
	for k, v := range query {
		q.Add(k, v)
	}
	r.URL.RawQuery = q.Encode()

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application/json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		return nil, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return []model.Tag{}, utils.StatusCodesToError(resp.StatusCode)
	}

	// Initialize a slice of Tag struct to store the response data.
	var tags []model.Tag

	// Decode the JSON response into the slice of Tag structs.
	err = json.NewDecoder(resp.Body).Decode(&tags)
	if err != nil {
		// Log the error and return an empty slice and the error.

		return []model.Tag{}, err
	}

	// Return the list of tags and no error (nil).
	return tags, nil
}
