package endpoints

import (
	"encoding/json"
	"fmt"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/model"
	"git.dragse.it/anthrove/e621-sdk-go/pkg/e621/utils"
	"net/http"
)

// GetUser retrieves user information from e621.net based on the provided username.
//
// Parameters:
//   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - username: The username of the user to retrieve.
//
// Returns:
//   - model.User: The retrieved user.
//   - error: An error, if any, encountered during the API request or response handling.
func GetUser(requestContext model.RequestContext, username string) (model.User, error) {
	// Create a new HTTP GET request to fetch user information from the specified 'host' and 'username'.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/users/%s.json", requestContext.Host, username), nil)
	if err != nil {
		// Log the error and return an empty User struct and the error.

		return model.User{}, err
	}

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application/json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		// Log the error and return an empty User struct and the error.

		return model.User{}, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return model.User{}, utils.StatusCodesToError(resp.StatusCode)
	}

	// Initialize a User struct to store the response data.
	var user model.User

	// Decode the JSON response into the User struct.
	err = json.NewDecoder(resp.Body).Decode(&user)
	if err != nil {
		// Log the error and return an empty User struct and the error.

		return model.User{}, err
	}

	// Return the user information and no error (nil).
	return user, nil
}

// GetUsers retrieves a list of users from e621.net based on query parameters.
//
// Parameters:
//   - requestContext: The context for the API request, including the host, user agent, username, and API key.
//   - query: A map containing additional query parameters for the API request.
//
// Returns:
//   - []model.User: A slice of users.
//   - error: An error, if any, encountered during the API request or response handling.
func GetUsers(requestContext model.RequestContext, query map[string]string) ([]model.User, error) {
	// Create a new HTTP GET request.
	r, err := http.NewRequest("GET", fmt.Sprintf("%s/users.json", requestContext.Host), nil)
	if err != nil {
		return nil, err
	}

	// Append query parameters to the request URL.
	q := r.URL.Query()
	for k, v := range query {
		q.Add(k, v)
	}
	r.URL.RawQuery = q.Encode()

	r.Header.Set("User-Agent", requestContext.UserAgent)
	r.Header.Add("Accept", "application/json")
	r.SetBasicAuth(requestContext.Username, requestContext.APIKey)

	// Send the request using the provided http.Client.
	resp, err := requestContext.Client.Do(r)
	if err != nil {
		return nil, err
	}

	// Check if the HTTP response status code indicates success (2xx range).
	if resp.StatusCode < 200 || resp.StatusCode > 300 {
		// If the status code is outside the 2xx range, return an error based on the status code.
		return []model.User{}, utils.StatusCodesToError(resp.StatusCode)
	}

	// Initialize a slice of User struct to store the response data.
	var users []model.User

	// Decode the JSON response into the slice of User structs.
	err = json.NewDecoder(resp.Body).Decode(&users)
	if err != nil {
		// Log the error and return an empty slice and the error.

		return []model.User{}, err
	}

	// Return the list of users and no error (nil).
	return users, nil
}
